/*
 * QUSOFT MICROSYSTMES
 * Assembly Studio for MVS
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Assembly Studio for MVS.
 *
 *  Assembly Studio for MVS is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Assembly Studio for MVS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Assembly Studio for MVS; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.System;//Import the class moka.lang.System
import moka.lang.String;//Import the class moka.lang.String

/**
 * Format OpCode R1,M3,D2(B2)
 *
 * @author  Frdric Brown
 */
public class FormatRS2 extends Format {

	private static FormatRS2 form;

    public String print (ubyte opCode, char *exprP) {
		NYBBLE R1;
		NYBBLE M3;
		BITS12 D2;
		NYBBLE B2;

		R1 = DisASM.getByte(exprP++, 4);
		M3 = DisASM.getByte(exprP++, 4);
		B2 = DisASM.getByte(exprP++, 4);

		D2 = DisASM.getHalf(exprP, 12);
		exprP += 3;

		String str = new String(DisASM.opVecStr[opCode]);

		str += " ";

		str += ((short)native.R1);

		str += ",B'";

		str += DisASM.printBin4((ubyte)M3);

		str += "',";

		str += ((short)native.D2);

		str += "(";

		str += ((short)native.B2);

		str += ")";

		return str;
	}

    public String assemble (ubyte opCode, char *exprP) {
		NYBBLE R1 = 0;
		NYBBLE R3 = 0;

		BITS12 D2 = 0;

		NYBBLE B2 = 0;

		R1 = DisASM.fetchRegister(&exprP);

		R3 = DisASM.fetchRegister(&exprP);

		D2 = DisASM.fetchD(&exprP);

		B2 = DisASM.fetchRegister(&exprP);

		String str = DisASM.printHex8(opCode);

		str += DisASM.printHex8((ubyte)R1, 1);

		str += DisASM.printHex8((ubyte)R3, 1);

		str += DisASM.printHex8((ubyte)B2, 1);

		str += DisASM.printHex12((short)D2);

		return str;
	}

	static {
		Format f = FormatRS2.form = new FormatRS2();
		char ** opVecStr = DisASM.opVecStr;
		Format[] opVecForm = DisASM.opVecForm;

		native {
			opVecStr[0xBD] = "CLM";
			opVecForm[0xBD] = f;

			opVecStr[0xBE] = "STCM";
			opVecForm[0xBE] = f;

			opVecStr[0xBF] = "ICM";
			opVecForm[0xBF] = f;
		}
	}

	finally {
		FormatRS2.form.finalize();
	}
}