/*
 * QUSOFT MICROSYSTMES
 * Assembly Studio for MVS
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Assembly Studio for MVS.
 *
 *  Assembly Studio for MVS is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Assembly Studio for MVS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Assembly Studio for MVS; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import moka.lang.System;//Import the class moka.lang.System
import moka.lang.String;//Import the class moka.lang.String

/**
 * Format OpCode R1,D2(X2,B2)
 *
 * @author  Frdric Brown
 */
public class FormatRX0 extends Format {

	private static FormatRX0 form;

    public String print (ubyte opCode, char *exprP) {
		NYBBLE R1;
		BITS12 D2;
		NYBBLE X2;
		NYBBLE B2;

		R1 = DisASM.getByte(exprP++, 4);

		X2 = DisASM.getByte(exprP++, 4);
		B2 = DisASM.getByte(exprP++, 4);

		D2 = DisASM.getHalf(exprP, 12);
		exprP += 3;

		String str = new String(DisASM.opVecStr[opCode]);

		str += " ";

		str += ((short)native.R1);

		str += ",";

		str += ((short)native.D2);

		str += "(";

		str += ((short)native.X2);

		str += ",";

		str += ((short)native.B2);

		str += ")";

		return str;
	}

    public String assemble (ubyte opCode, char *exprP) {
		NYBBLE R1 = 0;

		BITS12 D2 = 0;
		NYBBLE X2 = 0;
		NYBBLE B2 = 0;

		R1 = DisASM.fetchRegister(&exprP);

		D2 = DisASM.fetchD(&exprP);

		X2 = DisASM.fetchRegister(&exprP);

		B2 = DisASM.fetchRegister(&exprP);

		String str = DisASM.printHex8(opCode);

		str += DisASM.printHex8((ubyte)R1, 1);

		str += DisASM.printHex8((ubyte)X2, 1);

		str += DisASM.printHex8((ubyte)B2, 1);

		str += DisASM.printHex12((short)D2);

		return str;
	}

	static {
		Format f = FormatRX0.form = new FormatRX0();
		char ** opVecStr = DisASM.opVecStr;
		Format[] opVecForm = DisASM.opVecForm;

		native {
			opVecStr[0x40] = "STH";
			opVecForm[0x40] = f;

			opVecStr[0x41] = "LA";
			opVecForm[0x41] = f;

			opVecStr[0x42] = "STC";
			opVecForm[0x42] = f;

			opVecStr[0x43] = "IC";
			opVecForm[0x43] = f;

			opVecStr[0x44] = "EX";
			opVecForm[0x44] = f;

			opVecStr[0x45] = "BAL";
			opVecForm[0x45] = f;

			opVecStr[0x46] = "BCT";
			opVecForm[0x46] = f;

			opVecStr[0x47] = "BC";
			opVecForm[0x47] = f;

			opVecStr[0x48] = "LH";
			opVecForm[0x48] = f;

			opVecStr[0x49] = "CH";
			opVecForm[0x49] = f;

			opVecStr[0x4A] = "AH";
			opVecForm[0x4A] = f;

			opVecStr[0x4B] = "SH";
			opVecForm[0x4B] = f;

			opVecStr[0x4C] = "MH";
			opVecForm[0x4C] = f;

			opVecStr[0x4D] = "BAS";
			opVecForm[0x4D] = f;

			opVecStr[0x4E] = "CVD";
			opVecForm[0x4E] = f;

			opVecStr[0x4F] = "CVB";
			opVecForm[0x4F] = f;

			opVecStr[0x50] = "ST";
			opVecForm[0x50] = f;

			opVecStr[0x54] = "N";
			opVecForm[0x54] = f;

			opVecStr[0x55] = "CL";
			opVecForm[0x55] = f;

			opVecStr[0x56] = "O";
			opVecForm[0x56] = f;

			opVecStr[0x57] = "X";
			opVecForm[0x57] = f;

			opVecStr[0x58] = "L";
			opVecForm[0x58] = f;

			opVecStr[0x59] = "C";
			opVecForm[0x59] = f;

			opVecStr[0x5A] = "A";
			opVecForm[0x5A] = f;

			opVecStr[0x5B] = "S";
			opVecForm[0x5B] = f;

			opVecStr[0x5C] = "M";
			opVecForm[0x5C] = f;

			opVecStr[0x5D] = "D";
			opVecForm[0x5D] = f;

			opVecStr[0xD6] = "OC";
			opVecForm[0xD6] = f;
		}
	}

	finally {
		FormatRX0.form.finalize();
	}
}