/*
 * QUSOFT MICROSYSTMES
 * Assembly Studio for MVS
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Assembly Studio for MVS.
 *
 *  Assembly Studio for MVS is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Assembly Studio for MVS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Assembly Studio for MVS; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.System;//Import the class moka.lang.System
import moka.lang.String;//Import the class moka.lang.String

/**
 * Format OpCode D1(B1),I2
 *
 * @author  Frdric Brown
 */
public class FormatSI0 extends Format {

	private static FormatSI0 form;

    public String print (ubyte opCode, char *exprP) {
		BITS12 D1;
		NYBBLE B1;
		ubyte I2;

		I2 = DisASM.getByte(exprP, 8);
		exprP += 2;

		B1 = DisASM.getByte(exprP++, 4);

		D1 = DisASM.getHalf(exprP, 12);
		exprP += 3;

		String str = new String(DisASM.opVecStr[opCode]);

		str += " ";

		str += ((short)native.D1);

		str += "(";

		str += ((short)native.B1);

		str += "),X'";

		str += DisASM.printHex8(I2);

		str += "'";

		return str;
	}

    public String assemble (ubyte opCode, char *exprP) {
		ubyte I2 = 0;

		BITS12 D1 = 0;

		NYBBLE B1 = 0;

		D1 = DisASM.fetchD(&exprP);

		B1 = DisASM.fetchRegister(&exprP);

		I2 = DisASM.fetchValue(&exprP);


		String str = DisASM.printHex8(opCode);

		str += DisASM.printHex8(I2);

		str += DisASM.printHex8(B1, 1);

		str += DisASM.printHex12((short)D1);

		return str;
	}

	static {
		Format f = FormatSI0.form = new FormatSI0();
		char ** opVecStr = DisASM.opVecStr;
		Format[] opVecForm = DisASM.opVecForm;

		native {
			opVecStr[0x91] = "TM";
			opVecForm[0x91] = f;

			opVecStr[0x92] = "MVI";
			opVecForm[0x92] = f;

			opVecStr[0x94] = "NI";
			opVecForm[0x94] = f;

			opVecStr[0x95] = "CLI";
			opVecForm[0x95] = f;

			opVecStr[0x96] = "OI";
			opVecForm[0x96] = f;

			opVecStr[0x97] = "XI";
			opVecForm[0x97] = f;
		}
	}

	finally {
		FormatSI0.form.finalize();
	}
}