/*
 * QUSOFT MICROSYSTMES
 * Assembly Studio for MVS
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Assembly Studio for MVS.
 *
 *  Assembly Studio for MVS is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Assembly Studio for MVS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Assembly Studio for MVS; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.System;//Import the class moka.lang.System
import moka.lang.String;//Import the class moka.lang.String

/**
 * Format OpCode D1(L,B1),D2(B2)
 *
 * @author  Frdric Brown
 */
public class FormatSS0 extends Format {

	private static FormatSS0 form;

    public String print (ubyte opCode, char *exprP) {
		BITS12 D1;
		BYTE L;
		NYBBLE B1;
		BITS12 D2;
		NYBBLE B2;

		L = DisASM.getByte(exprP, 8);
		exprP += 2;

		B1 = DisASM.getByte(exprP++, 4);

		D1 = DisASM.getHalf(exprP, 12);
		exprP += 3;

		B2 = DisASM.getByte(exprP++, 4);

		D2 = DisASM.getHalf(exprP, 12);
		exprP += 3;

		String str = new String(DisASM.opVecStr[opCode]);

		str += " ";

		str += ((short)native.D1);

		str += "(";

		str += ((short)native.++L);

		str += ",";

		str += ((short)native.B1);

		str += "),";

		str += ((short)native.D2);

		str += "(";

		str += ((short)native.B2);

		str += ")";

		return str;
	}

    public String assemble (ubyte opCode, char *exprP) {
		BITS12 D1;
		ubyte L;
		NYBBLE B1;
		BITS12 D2;
		NYBBLE B2;

		D1 = DisASM.fetchD(&exprP);

		L  = (BYTE)DisASM.fetchL(&exprP);

		B1 = DisASM.fetchRegister(&exprP);

		D2 = DisASM.fetchD(&exprP);

		B2 = DisASM.fetchRegister(&exprP);

		String str = DisASM.printHex8(opCode);

		str += DisASM.printHex8(--L);

		str += DisASM.printHex8((ubyte)B1, 1);

		str += DisASM.printHex12((short)D1);

		str += DisASM.printHex8((ubyte)B2, 1);

		str += DisASM.printHex12((short)D2);

		return str;
	}

	static {
		Format f = FormatSS0.form = new FormatSS0();
		char ** opVecStr = DisASM.opVecStr;
		Format[] opVecForm = DisASM.opVecForm;

		native {
			opVecStr[0xD2] = "MVC";
			opVecForm[0xD2] = f;

			opVecStr[0xD4] = "NC";
			opVecForm[0xD4] = f;

			opVecStr[0xD5] = "CLC";
			opVecForm[0xD5] = f;

			opVecStr[0xD6] = "OC";
			opVecForm[0xD6] = f;

			opVecStr[0xD7] = "XC";
			opVecForm[0xD7] = f;

			opVecStr[0xDC] = "TR";
			opVecForm[0xDC] = f;

			opVecStr[0xDD] = "TRT";
			opVecForm[0xDD] = f;
		}
	}

	finally {
		FormatSS0.form.finalize();
	}
}