/*
 * QUSOFT MICROSYSTMES
 * Assembly Studio for MVS
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Assembly Studio for MVS.
 *
 *  Assembly Studio for MVS is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Assembly Studio for MVS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Assembly Studio for MVS; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.System;
import moka.lang.String;
import moka.x.Frame;
import moka.x.GEM;
import moka.x.Font;
import moka.x.Button;
import moka.x.TextArea;
import moka.x.Font;
import moka.io.File;
import moka.x.OptionPane;

/**
 * This class is the main class of Assembly Studio for MVS.
 *
 * @author  Frdric Brown
 */
public class GuiASM extends Frame {

	private Button btnDis;
	private TextArea txtCode;
	private FrameASM frameASM;
	private FrameAbout frameAbout;

	private	MenuBar bar;
	private  	Menu mFile;
	private		MenuItem itmOpenASM;
	private		MenuItem itmSaveASM;
	private		MenuItem itmOpenBIN;
	private		MenuItem itmSaveBIN;
	private		MenuItem itmQuit;

	private Menu mView;
	private		MenuItem itmCode;
	private		MenuItem itmASM;

	private	Menu mHelp;
	private		MenuItem itmAbout;

	public GuiASM () {
		super("Machine code");

		if (System.CALCULATOR) {
			this.setBounds(0, 0, 200, 100);
		}
		else {
			this.setBounds(0, 0, 120, 90);
		}

		this.defaultCloseOperation = Frame.HIDE_ON_CLOSE;

		this.btnDis = new Button("DeASM");
		this.btnDis.setBounds(5, this.height - 20, 40, 15);
		this.btnDis.mnemonic = 'D';
		this.btnDis.listen = this;
		this.add(this.btnDis);

		this.txtCode = new TextArea();
		if (System.CALCULATOR) {
			this.txtCode.setBounds(5, 15, 190, 60);
		}
		else {
			this.txtCode.setBounds(5, 15, 110, 50);
		}

		this.txtCode.warp = false;
		this.txtCode.vScrollVisible = true;
		this.add(this.txtCode);

		this.frameASM = new FrameASM(this);
		this.frameAbout = new FrameAbout();

		this.bar = new MenuBar();
		this.bar.setBounds(0, 0, GEM.LCD_WIDTH - 2, 15);
		GEM.gem.add(this.bar);
			this.mFile = new Menu("File");
			this.mFile.mnemonic = 'F';
			this.bar.add(this.mFile);
				this.itmOpenASM = new MenuItem("Import ASM");
				this.itmOpenASM.mnemonic = 'M';
				this.itmOpenASM.listen = this;
				this.mFile.add(this.itmOpenASM);
				this.itmSaveASM = new MenuItem("Export ASM");
				this.itmSaveASM.mnemonic = 'X';
				this.itmSaveASM.listen = this;
				this.mFile.add(this.itmSaveASM);
				this.itmOpenBIN = new MenuItem("Import code");
				this.itmOpenBIN.mnemonic = 'I';
				this.itmOpenBIN.listen = this;
				this.mFile.add(this.itmOpenBIN);
				this.itmSaveBIN = new MenuItem("Export code");
				this.itmSaveBIN.mnemonic = 'E';
				this.itmSaveBIN.listen = this;
				this.mFile.add(this.itmSaveBIN);
				this.itmQuit = new MenuItem("Quit");
				this.itmQuit.mnemonic = 'Q';
				this.itmQuit.listen = this;
				this.mFile.add(this.itmQuit);
			this.mView = new Menu("View");
			this.mView.mnemonic = 'V';
			this.bar.add(this.mView);
				this.itmCode = new MenuItem("Machine code");
				this.itmCode.mnemonic = 'M';
				this.itmCode.listen = this;
				this.mView.add(this.itmCode);
				this.itmASM = new MenuItem("Assembler");
				this.itmASM.mnemonic = 'A';
				this.itmASM.listen = this;
				this.mView.add(this.itmASM);
			this.mHelp = new Menu("Help");
			this.mHelp.mnemonic = 'H';
			this.bar.add(this.mHelp);
				this.itmAbout = new MenuItem("About...");
				this.itmAbout.mnemonic = 'A';
				this.itmAbout.listen = this;
				this.mHelp.add(this.itmAbout);

		GEM.gem.add(this);
		GEM.gem.add(this.frameASM);
		GEM.gem.add(this.frameAbout);
		this.frameAbout.center();
	}

	public void eventTriggered(Object sender) {
		File f;
		String s;
		char c;
		char[] str;

		if (sender == this.btnDis) {
			this.frameASM.setASMFromCode(this.txtCode.caption);
			this.frameASM.setVisible(true);
			GEM.gem.toFront(this.frameASM);
		}
		else if (sender == this.itmQuit) {
			GEM.gem.setStarted(false);
		}
		else if (sender == this.itmCode) {
			this.setVisible(true);
			GEM.gem.toFront(this);
		}
		else if (sender == this.itmASM) {
			this.frameASM.setVisible(true);
			GEM.gem.toFront(this.frameASM);
		}
		else if (sender == this.itmAbout) {
			this.frameAbout.setVisible(true);
			GEM.gem.toFront(this.frameAbout);
		}
		else if (sender == this.itmOpenASM || sender == this.itmOpenBIN) {
			f = File.showVarLinkDialog();

			if (f) {

				f.open(File.MODE_OPEN_READ_TEXT);

				if (f.error) {
					f.finalize();
					return;
				}

				str = new char[4096];

				str[f.readBytesCount(str, f.available())] = '\0';
				
				s = new String(str, false);

				if (sender == this.itmOpenASM) {
					this.frameASM.txtASM.setCaption(s);
					this.frameASM.setVisible(true);
					GEM.gem.toFront(this.frameASM);
				}
				else {
					this.txtCode.setCaption(s);
					this.setVisible(true);
					GEM.gem.toFront(this);
				}

				f.finalize();
			}
		}
		else if (sender == this.itmSaveASM || sender == this.itmSaveBIN) {
			s = OptionPane.showInputDialog("File name");

			if (s) {
				f = new File(s);

				f.open(File.MODE_WRITE_TEXT);

				if (!f.error) {
					if (sender == this.itmSaveASM) {
						f.writeChars(this.frameASM.txtASM.caption.value);
					}
					else {
						f.writeChars(this.txtCode.caption.value);
					}
				}

				f.finalize();
			}
		}

	}

	public void setCodeFromASM(String asmStr) {
		String code = "";

		char[] exprP = asmStr.value;
		char inst[25];
		short i;

		while (*exprP) {

			while (((*exprP == ' ') || (*exprP == '\n')) && *exprP) exprP++;

			for (i = 0; exprP[0] != ' ' && exprP[0] != '\n' && exprP[0]; i++) {
				inst[i] = *exprP++;
			}
			
			if (i > 0) {
				inst[i] = '\0';
			
				i = DisASM.trouverIndex(inst);
	
				if (i >= 0) {
					code += DisASM.opVecForm[i].assemble((ubyte)i, exprP);
				}
			}

			while (*exprP != '\n' && *exprP) exprP++;
		}

		this.txtCode.setCaption(code);
	}

	public static void main () {
		GEM.gem.setFont(Font.FONT_6x8);

		System.clrscr();
		GEM.gem.setBackground();

		GuiASM f = new GuiASM();
	}
}