//Defining class Character
/*
 * This file contains the definition of all the methods of the class and the definition of the built in functions related to the class.
 */
/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */
/*
 *  This file is part of Moka API.
 *
 *  Moka API is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka API is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Moka API; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * The <code>Character</code> class wraps a value of the primitive
 * type <code>char</code> in an object. An object of type
 * <code>Character</code> contains a single field whose type is
 * <code>char</code>.
 * <p>
 * In addition, this class provides several methods for determining
 * a character's category (lowercase letter, digit, etc.) and for converting
 * characters from uppercase to lowercase and vice versa. Plus, this class offers
 * several method for ANSI string manipulation.
 * <p>
 * Character information is based on the ASCII.
 * @author  Frdric Brown
 * @version 1.0, 2002-08-29
 * @since   MDK1.0a
 */
/**
     * Concatenates another ANSI string to the end of the specified ANSI string.
     * <p>
     * If the length of the argument string is <code>0</code>, then this
     * ANSI String object is copied then returned. Otherwise, a new
     * ANSI String object is created, representing a character
     * sequence that is the concatenation of the character sequence
     * represented by this ANSI String object and the character
     * sequence represented by the argument string.<p>
     *
     * @param   ansiString   the ANSI String that will be followed by the other ANSI string.
     * @param   anotherAnsiString   the ANSI String that is concatenated to the end
     *                of the specified ANSI String.
     * @return  an ANSI string that represents the concatenation of the specified ANSI string
     *          characters followed by the other ANSI string argument's characters.
     */
char*  Character_concat_char_p_char_p ( char * ansiString , char * anotherAnsiString ) { 
long int len0 = strlen(ansiString);
		long int len1 = strlen(anotherAnsiString);
		char* str = malloc(len0 + len1 + 1);
		strcpy(str, ansiString);
		return strcat(str, anotherAnsiString);
	
} 
/**
     * Returns a copy of the ANSI string
     * argument. The contents of the character array are copied; subsequent
     * modification of the character array does not affect the newly
     * created ANSI string.
     *
     * @param   data   an ANSI string.
     * @return  a newly allocated  ANSI string representing the same sequence of
     *          characters contained in the ANSI string argument.
     */
char*  Character_copyValueOf_char_p ( char * ansiString ) { 
return strcpy(malloc(strlen(ansiString) + 1), ansiString);
	
} 
/**
     * Determines if the specified character is a digit.
	 *
     * @param   ch   the character to be tested.
     * @return  <code>true</code> if the character is a digit;
     *          <code>false</code> otherwise.
     */
BOOL Character_isDigit_char ( char ch ) { 
	return ( ch >= '0' && ch <= '9' ) ; 
	
} 
/**
     * Determines if the specified character is a letter.
     *
     * @param   ch   the character to be tested.
     * @return  <code>true</code> if the character is a letter;
     *          <code>false</code> otherwise.
     */
BOOL Character_isLetter_char ( char ch ) { 
	return Character_isLowerCase_char ( ch ) || Character_isUpperCase_char ( ch ) ; 
	
} 
/**
     * Determines if the specified character is a lowercase character.
     * <p>
     * The following are examples of lowercase characters:
     * <p><blockquote><pre>
     * a b c d e f g h i j k l m n o p q r s t u v w x y z
     * </pre></blockquote>
     *
     * @param   ch   the character to be tested.
     * @return  <code>true</code> if the character is lowercase;
     *          <code>false</code> otherwise.
     */
BOOL Character_isLowerCase_char ( char ch ) { 
return ch >= 'a' && ch <= 'z' || (unsigned char)ch >= 224 && (unsigned char)ch <= 246 || (unsigned char)ch >= 248 && (unsigned char)ch <= 253;
    
} 
/**
     * Determines if the specified character is an uppercase character.
     * <p>
     * The following are examples of uppercase characters:
     * <p><blockquote><pre>
     * A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
     * </pre></blockquote>
     *
     * @param   ch   the character to be tested.
     * @return  <code>true</code> if the character is uppercase;
     *          <code>false</code> otherwise.
     */
BOOL Character_isUpperCase_char ( char ch ) { 
return ch >= 'A' && ch <= 'Z' || (unsigned char)ch >= 192 && (unsigned char)ch <= 214 || (unsigned char)ch >= 216 && (unsigned char)ch <= 221;
    
} 
/**
     * Converts the character argument to lowercase using case
     * mapping information from the TI-92 Plus Character Codes chart.
     *
     * @param   ch   the character to be converted.
     * @return  the lowercase equivalent of the character, if any;
     *          otherwise, the character itself.
     */
char Character_toLowerCase_char ( char ch ) { 
if (ch >= 'A' && ch <= 'Z' || (unsigned char)ch >= 192 && (unsigned char)ch <= 214 || (unsigned char)ch >= 216 && (unsigned char)ch <= 221) {
    		ch = ch + 32;
    	}

    	return ch;
    
} 
