//Defining class Container
/*
 * This file contains the definition of all the methods of the class and the definition of the built in functions related to the class.
 */
/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */
/*
 *  This file is part of Moka API.
 *
 *  Moka API is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka API is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Moka API; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Container is a component that contains child components.
 *
 * @author  Frdric Brown
 * @version 1.0, 2002-10-31
 * @since   MDK1.0a
 */
TContainer * TContainer0 ( ) {
	TContainer * this = malloc ( sizeof ( TContainer ) ) ; 
	this -> class = ( char * ) "Container" ; 
	this -> class_id = 20 ; 
	this -> super = ( void * ) Component ; 
	//Methods from class Object
	this -> toString_ = ( void * ) TObject_toString_ ; 
	//Methods from class EventListener
	//Methods from class Component
	this -> finalize_ = ( void * ) TComponent_finalize_ ; 
	this -> eventTriggered_Object = ( void * ) TComponent_eventTriggered_Object ; 
	this -> paint_ = ( void * ) TComponent_paint_ ; 
	this -> setBounds_short_int_short_int_short_int_short_int = ( void * ) TComponent_setBounds_short_int_short_int_short_int_short_int ; 
	this -> setY_short_int = ( void * ) TComponent_setY_short_int ; 
	this -> move_short_int_short_int = ( void * ) TComponent_move_short_int_short_int ; 
	this -> setVisible_BOOL = ( void * ) TComponent_setVisible_BOOL ; 
	this -> isCursorOver_short_int_short_int = ( void * ) TComponent_isCursorOver_short_int_short_int ; 
	this -> center_ = ( void * ) TComponent_center_ ; 
	//Methods from class Container
	this -> onPaint_ = ( void * ) TContainer_onPaint_ ; 
	this -> onRefresh_ = ( void * ) TContainer_onRefresh_ ; 
	this -> repaintChildren_ = ( void * ) TContainer_repaintChildren_ ; 
	this -> paintForeground_ = ( void * ) TContainer_paintForeground_ ; 
	this -> paintChildren_SCR_RECT_p = ( void * ) TContainer_paintChildren_SCR_RECT_p ; 
	this -> use_ = ( void * ) TContainer_use_ ; 
	this -> checkMnemonic_char = ( void * ) TContainer_checkMnemonic_char ; 
	this -> add_Component = ( void * ) TContainer_add_Component ; 
	this -> remove_Component = ( void * ) TContainer_remove_Component ; 
	this -> toFront_Component = ( void * ) TContainer_toFront_Component ; 
	this -> clean_ = ( void * ) TContainer_clean_ ; 
	return this ; 
} 
/**
	 * Constructs a newly allocated Container.
	 */
TContainer * TContainer_ ( ) {
	TContainer * this = TContainer0 ( ) ; 
	Container__ ( this ) ; 
	return this ; 
} 
void Container__ ( struct Container * this ) { 
	Component__ ( ( TComponent *  ) this ) ; 
	this -> child = NULL ; 
	this -> tail = NULL ; 
	
} 
/**
	 * Called when a PaintEvent occurs to paint the image of the container and
	 * stores it in its virtual memory.  The container is painted, then all its
	 * children are and finally its foreground is.  A PaintEvent occurs when the
	 * aspect of the container is altered or any of its children is.
	 */
void TContainer_onPaint_ ( struct Container * this ) { 
	this -> repaint = FALSE ; 
	this -> repaintChildren_ ( this ) ; 
	SCR_RECT rect = (SCR_RECT){{0, 0, this->width - 1, this->height - 1}};

			PortSet(GEM_gem->port, 239, 127);
			ScrRectFill (&rect, &rect, A_REVERSE);
		 this -> paint_ ( this ) ; 
	this -> paintChildren_SCR_RECT_p ( this , ( SCR_RECT * ) & rect ) ; 
	this -> paintForeground_ ( this ) ; 
	if (this->img) {
				free (this->img);
			}

			this->img = malloc(BitmapSize(&rect));
			BitmapGet (&rect, this->img);
			PortSet ((void *) 0x4C00, 239, 127);
		 
} 
/**
	 * Called when an UpdateEvent occurs.  An update event occurs when a
	 * visible component contained in this container has changed its
	 * appearance.
	 */
void TContainer_onRefresh_ ( struct Container * this ) { 
	this -> repaint = TRUE ; 
	if ( this -> visible && this -> parent ) { 
		this -> parent -> onRefresh_ ( this -> parent ) ; 
		
	} 
	
} 
/**
	 * Sends a PaintEvent to all the children of this component
	 * that need to be repainted.
	 */
void TContainer_repaintChildren_ ( struct Container * this ) { 
	TComponent *  c ; 
	c = this -> child ; 
	while ( c ) { 
		if ( c -> visible && c -> repaint ) { 
			c -> onPaint_ ( c ) ; 
			
		} 
		c = c -> next ; 
		
	} 
	
} 
/**
	 * This method is called to draw the foreground of the
	 * container. This method is called after all the children
	 * of this container are painted.
	 */
void TContainer_paintForeground_ ( struct Container * this ) { 
	
} 
/**
	 * Paints the children of this container to the virtual screen.
	 *
	 * param clip The SCR_RECT clip
	 */
void TContainer_paintChildren_SCR_RECT_p ( struct Container * this , SCR_RECT * clip ) { 
	TComponent *  c ; 
	c = this -> child ; 
	while ( c ) { 
		if ( c -> visible ) { 
			BitmapPut (c->x, c->y, c->img, clip, A_REPLACE);
				 
		} 
		c = c -> next ; 
		
	} 
	
} 
/**
	 * This method is invoked by the GEM when the user interacts
	 * with the container.
	 */
void TContainer_use_ ( struct Container * this ) { 
	TComponent *  c ; 
	short int x = 0 ; 
	short int y = 0 ; 
	TContainer *  ct = this ; 
	if ( ! this -> enabled ) { 
		return ; 
		
	} 
	while ( ct ) { 
		x = x + ct -> x ; 
		y = y + ct -> y ; 
		ct = ct -> parent ; 
		
	} 
	c = this -> tail ; 
	while ( c ) { 
		if ( c -> visible && c -> isCursorOver_short_int_short_int ( c , x , y ) ) { 
			c -> use_ ( c ) ; 
			return ; 
			
		} 
		c = c -> prev ; 
		
	} 
	if ( this -> listen ) { 
		this -> listen -> eventTriggered_Object ( this -> listen , this ) ; 
		
	} 
	
} 
/**
	 * Checks if the mnemonic of this container correspond
	 * to the current mnemonic. If it is the case, an event
	 * is triggered.
	 */
void TContainer_checkMnemonic_char ( struct Container * this , char m ) { 
	TComponent *  c = this -> tail ; 
	if ( this -> mnemonic == m ) { 
		if ( this -> listen ) { 
			this -> listen -> eventTriggered_Object ( this -> listen , this ) ; 
			
		} 
		return ; 
		
	} 
	while ( c ) { 
		if ( c -> visible ) { 
			c -> checkMnemonic_char ( c , m ) ; 
			
		} 
		c = c -> prev ; 
		
	} 
	
} 
/**
	 * Adds the specified component to this container.
	 *
	 * @param c the component to add to this container.
	 */
void TContainer_add_Component ( struct Container * this , struct Component * c ) { 
	if ( ! this -> child ) { 
		this -> child = c ; 
		
	} 
	if ( this -> tail ) { 
		this -> tail -> next = c ; 
		
	} 
	c -> prev = this -> tail ; 
	c -> next = NULL ; 
	this -> tail = c ; 
	c -> parent = this ; 
	if ( c -> visible ) { 
		this -> repaint = TRUE ; 
		if ( this -> visible && this -> parent ) { 
			this -> parent -> onRefresh_ ( this -> parent ) ; 
			
		} 
		
	} 
	
} 
/**
	 * Removes the specified component from this container.
	 *
	 * @param c the component to remove from this container
	 */
void TContainer_remove_Component ( struct Container * this , struct Component * c ) { 
	TComponent *  cpn = this -> child ; 
	while ( cpn ) { 
		if ( cpn == c ) { 
			if ( c -> prev ) { 
				c -> prev -> next = c -> next ; 
				
			} 
			if ( c -> next ) { 
				c -> next -> prev = c -> prev ; 
				
			} 
			if ( this -> child == c ) { 
				this -> child = c -> next ; 
				
			} 
			if ( this -> tail == c ) { 
				this -> tail = c -> prev ; 
				
			} 
			c -> parent = NULL ; 
			c -> next = NULL ; 
			c -> prev = NULL ; 
			break ; 
			
		} 
		cpn = cpn -> next ; 
		
	} 
	if ( c -> visible ) { 
		this -> repaint = TRUE ; 
		if ( this -> visible && this -> parent ) { 
			this -> parent -> onRefresh_ ( this -> parent ) ; 
			
		} 
		
	} 
	
} 
/**
	 * Called to brings a component to the front
	 * of this container.
	 *
	 * @param c The component to bring to front.
	 * @return This container if it is the component to bring to front or if a child of
	 * 			this container is, null otherwize.
	 */
TComponent *  TContainer_toFront_Component ( struct Container * this , struct Component * c ) { 
	TComponent *  cpn = this -> child ; 
	if ( this == c ) { 
		return this ; 
		
	} 
	while ( cpn ) { 
		if ( cpn == cpn -> toFront_Component ( cpn , c ) ) { 
			if ( cpn != this -> tail ) { 
				this -> remove_Component ( this , cpn ) ; 
				this -> add_Component ( this , cpn ) ; 
				if ( c -> visible ) { 
					this -> repaint = TRUE ; 
					if ( this -> visible && this -> parent ) { 
						this -> parent -> onRefresh_ ( this -> parent ) ; 
						
					} 
					
				} 
				
			} 
			return this ; 
			
		} 
		cpn = cpn -> next ; 
		
	} 
	return NULL ; 
	
} 
/**
	 * Disposes of all ressources used by this container and
	 * all its children.
	 */
void TContainer_clean_ ( struct Container * this ) { 
	TComponent *  c ; 
	TComponent *  cn = this -> child ; 
	while ( cn ) { 
		c = cn ; 
		cn = cn -> next ; 
		c -> clean_ ( c ) ; 
		
	} 
	this -> finalize_ ( this ) ; 
	
} 
