//Defining class StringTokenizer
/*
 * This file contains the definition of all the methods of the class and the definition of the built in functions related to the class.
 */
/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */
/*
 *  This file is part of Moka API.
 *
 *  Moka API is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka API is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Moka API; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * The string tokenizer class allows an application to break a
 * string into tokens.
 * The delimiters (the characters that separate tokens) may
 * be specified either at creation time or on a per-token basis.
 *
 * @author  Frdric Brown
 * @version 1.0, 2002-11-08
 * @since   MDK1.0a
 */
TStringTokenizer * TStringTokenizer0 ( ) {
	TStringTokenizer * this = malloc ( sizeof ( TStringTokenizer ) ) ; 
	this -> class = ( char * ) "StringTokenizer" ; 
	this -> class_id = 32 ; 
	this -> super = ( void * ) Object ; 
	//Methods from class Object
	this -> toString_ = ( void * ) TObject_toString_ ; 
	//Methods from class StringTokenizer
	this -> finalize_ = ( void * ) TStringTokenizer_finalize_ ; 
	this -> nextToken_ = ( void * ) TStringTokenizer_nextToken_ ; 
	this -> setCurrentPosition_long_int = ( void * ) TStringTokenizer_setCurrentPosition_long_int ; 
	return this ; 
} 
/**
     * Constructs a string tokenizer for the specified string. The
     * tokenizer uses the default delimiters, which is
     * the space character,
     * the tab character, the newline character, the carriage-return character,
     * and the form-feed character.
     *
     * @param   str   a string to be parsed.
     */
TStringTokenizer * TStringTokenizer_String ( struct String * str ) {
	TStringTokenizer * this = TStringTokenizer0 ( ) ; 
	this -> str = NULL  ; 
	this -> origin = NULL  ; 
	StringTokenizer__String ( this , str ) ; 
	return this ; 
} 
void StringTokenizer__String ( struct StringTokenizer * this , struct String * str ) { 
//TAB, NL, FF, CR, SPACE
    	char delim[] = {9, 10, 12, 13, 32, '\0'};
    	TString* dstr = TString_char_p(delim);

    	StringTokenizer__String_String(this, str, dstr);

    	//dstr->finalize_(dstr);
    
} 
/**
     * Constructs a string tokenizer for the specified string. The
     * characters in the <code>delim</code> argument is the delimiters
     * for separating tokens.
     *
     * @param   str     a string to be parsed.
     * @param   delim   the delimiter.
     */
TStringTokenizer * TStringTokenizer_String_String ( struct String * str , struct String * delim ) {
	TStringTokenizer * this = TStringTokenizer0 ( ) ; 
	this -> str = NULL  ; 
	this -> origin = NULL  ; 
	StringTokenizer__String_String ( this , str , delim ) ; 
	return this ; 
} 
void StringTokenizer__String_String ( struct StringTokenizer * this , struct String * str , struct String * delim ) { 
	long int len = ( strlen(delim->value)  ) ; 
	long int i ; 
	this -> str = str ; 
	this -> origin = str -> toString_ ( str ) ; 
	this -> setCurrentPosition_long_int ( this , 0 ) ; 
	for ( i = 0 ; 
	i < len ; 
	i ++ ) { 
		str -> replace_char_char ( str , ( char ) delim -> value [ i ] , ( char ) 1 ) ; 
		
	} 
	delim -> finalize_ ( delim ) ; 
	
} 
/**
     * Frees the memory of system ressources used by this StringTokenizer
     * object.
     *
     */
void TStringTokenizer_finalize_ ( struct StringTokenizer * this ) { 
if (this->str) {
 			this->str->finalize_(this->str);
 			this->origin->finalize_(this->origin);
 		}
 		free(this);
 	
} 
/**
     * Returns the next token from this string tokenizer.
     *
     * @return     the next token from this string tokenizer.
     */
TString *  TStringTokenizer_nextToken_ ( struct StringTokenizer * this ) { 
	long int len = ( strlen(this->str->value)  ) ; 
	long int old = this -> currentPosition ; 
	long int end ; 
	char * begin = ( &this->str->value[old]  ) ; 
	char * pos = ( strchr(begin, 1)  ) ; 
	if ( pos ) { 
		this -> currentPosition = end = old + ( long int ) ( pos - begin ) ; 
		
	} 
	else { 
		this -> currentPosition = end = len ; 
		
	} 
	this -> setCurrentPosition_long_int ( this , this -> currentPosition ) ; 
	return this -> str -> substring_long_int_long_int ( this -> str , old , end ) ; 
	
} 
/**
     * Sets the current position to the specified index.
     *
     * currentPosition the new current position.
     */
void TStringTokenizer_setCurrentPosition_long_int ( struct StringTokenizer * this , long int currentPosition ) { 
	long int len = ( strlen(this->str->value)  ) ; 
	this -> currentPosition = currentPosition ; 
	while ( this -> currentPosition < len && this -> str -> value [ this -> currentPosition ] == 1 ) this -> currentPosition ++ ; 
	
} 
