//Defining class Double
/*
 * This file contains the definition of all the methods of the class and the definition of the built in functions related to the class.
 */
/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */
/*
 *  This file is part of Moka API.
 *
 *  Moka API is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka API is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Moka API; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * The <code>Double</code> class wraps a value of the primitive type
 * <code>double</code> in an object. An object of type
 * <code>Double</code> contains a single field whose type is
 * <code>double</code>.
 * <p>
 * In addition, this class provides several methods for converting a
 * <code>double</code> to a <code>String</code> and a
 * <code>String</code> to a <code>double</code>, as well as other
 * constants and methods useful when dealing with a
 * <code>double</code>.
 * <p>
 * This class can also be used to manipulate float.
 *
 * @author  Frdric Brown
 * @version 1.0, 2002-09-05
 * @since   MDK1.0a
 */
/**
     * Parses the string argument as a signed decimal integer.  It recognizes
     * the character representation of a floating point number. The resulting
     * double value is returned.  The string argument is finalized before the method
     * returns.
     *
     * @param s	   a <code>String</code> containing the <code>double</code>
     *             representation to be parsed
     * @return     the double value represented by the argument in decimal.
     */
double Double_parseDouble_String ( struct String * s ) { 
char* buffer = Character_copyValueOf_char_p(s->value);
		short i;
		double value;
		char c;

		for (i = 0; (c = buffer[i]); i++)
		{
			if (c == '-') buffer[i] = 0xAD;
			if ((c|32) == 'e') buffer[i] = 0x95;
		}

		value = atof(buffer);

		free(buffer);

		s->finalize_(s);

    	return value;
    
} 
/**
     * Checks if the specified string is a decimal representation
     * of a double.
     *
     * @param str The string to verify
     * @return true if the specified string is a decimal representation of a double, false otherwize.
     */
BOOL Double_isDouble_String ( struct String * str ) { 
	return Double_parseDouble_String ( str -> toString_ ( str ) ) != NAN ; 
	
} 
