//Defining class File
/*
 * This file contains the definition of all the methods of the class and the definition of the built in functions related to the class.
 */
/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002-2003 Frdric Brown
 */
/*
 *  This file is part of Moka API.
 *
 *  Moka API is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka API is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Moka API; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * An abstract representation of file and directory pathnames.
 *
 * @author  Frdric Brown
 * @version 1.1, 2003-11-02
 * @since   MDK1.0a
 */
TFile * TFile0 ( ) {
	TFile * this = malloc ( sizeof ( TFile ) ) ; 
	this -> class = ( char * ) "File" ; 
	this -> class_id = 28 ; 
	this -> super = ( void * ) IOStream ; 
	//Methods from class Object
	this -> toString_ = ( void * ) TObject_toString_ ; 
	//Methods from class IOStream
	this -> writeChars_char_p = ( void * ) TIOStream_writeChars_char_p ; 
	//Methods from class File
	this -> open_short_int = ( void * ) TFile_open_short_int ; 
	this -> close_ = ( void * ) TFile_close_ ; 
	this -> available_ = ( void * ) TFile_available_ ; 
	this -> writeBytes_char_p_short_int = ( void * ) TFile_writeBytes_char_p_short_int ; 
	this -> readBytesCount_char_p_short_int = ( void * ) TFile_readBytesCount_char_p_short_int ; 
	this -> finalize_ = ( void * ) TFile_finalize_ ; 
	return this ; 
} 
/**
     * Creates a new <code>File</code> instance by using the given pathname string.
     * If the pathname string object risk to be used later, use a copy of the string object
     * instead of the original.
     *
     @param pathname A pathname string
     */
TFile * TFile_String ( struct String * pathname ) {
	TFile * this = TFile0 ( ) ; 
	this -> name = NULL  ; 
	this -> type = NULL  ; 
	File__String ( this , pathname ) ; 
	return this ; 
} 
void File__String ( struct File * this , struct String * pathname ) { 
	IOStream__ ( ( TIOStream *  ) this ) ; 
	this -> name = pathname ; 
	this -> file = NULL ; 
	this -> mode = File_MODE_OPEN_READ ; 
	
} 
/**
	 * Shows the VAR-LINK dialog and return a file object, associated to the TIOS variable selected in the dialog.
	 *
	 * @return a file object associated to the TIOS variable selected in the dialog, or null if no file was selected.
	 */
TFile *  File_showVarLinkDialog_ ( ) { 
EVENT ev;
		File_varBuffer = (char*)malloc(sizeof(char) * 20);
		TString* str;
		short len;

		File_varBuffer[0] = 0;

  		EVENT_HANDLER OldHandler = EV_captureEvents (VarLinkHandler);
  		memset (&ev, sizeof (ev), 0);
  		ev.Type = CM_KEYPRESS;
  		ev.extra.Key.Code = KEY_VARLNK;
  		EV_defaultHandler (&ev);
  		EV_captureEvents (OldHandler);

		if (strcmp("", File_varBuffer)) {
			str = TString_char_p_BOOL(File_varBuffer, FALSE);

			len = strlen(str->value);

			if (str->value[len-1] == '(') {
				str->delete_long_int_long_int(str, len-1, len);
			}

			return TFile_String(str);
		}
		else {
			free(File_varBuffer);
			return NULL;
		}
	
} 
/**
	 * Opens a binary file.
	 @param mode The file mode
	 */
void TFile_open_short_int ( struct File * this , short int mode ) { 
char* m = NULL;

		this->mode = mode;

		switch (mode) {
			case File_MODE_OPEN_READ: {
				m = (char*) "rb";
				break;
			}
			case File_MODE_WRITE: {
				m = (char*) "wb";
				break;
			}
			case File_MODE_APPEND: {
				m = (char*) "ab";
				break;
			}
			case File_MODE_OPEN_UPDATE: {
				m = (char*) "r+b";
				break;
			}
			case File_MODE_UPDATE: {
				m = (char*) "w+b";
				break;
			}
			case File_MODE_UPDATE_APPEND: {
				m = (char*) "a+b";
				break;
			}
			case File_MODE_OPEN_READ_TEXT: {
				m = (char*) "r";
				break;
			}
			case File_MODE_WRITE_TEXT: {
				m = (char*) "w";
				break;
			}
			case File_MODE_APPEND_TEXT: {
				m = (char*) "a";
				break;
			}
			case File_MODE_OPEN_UPDATE_TEXT: {
				m = (char*) "r+";
				break;
			}
			case File_MODE_UPDATE_TEXT: {
				m = (char*) "w+";
				break;
			}
			case File_MODE_UPDATE_APPEND_TEXT: {
				m = (char*) "a+";
				break;
			}
		}

		this->file = fopen(this->name->value, m);

		this->error = (this->file == NULL);

		this->opened = !this->error;

	
} 
/**
	 * Closes the stream.
	 */
void TFile_close_ ( struct File * this ) { 
	if ( this -> type ) { 
		fputc (0, this->file);
				fputs (this->type->value, this->file);
				fputc (0, this->file);
				fputc (OTH_TAG, this->file);
			 
	} 
	this -> error = ( fclose(this->file)  ) ; 
	this -> file = NULL ; 
	this -> opened = FALSE ; 
	
} 
/**
	 * Returns the number of bytes that can be read from this file.
	 *
	 * @return	the number of bytes that can be read from this file.
	 */
long int TFile_available_ ( struct File * this ) { 
long int num = (this->file->alloc - (this->file->fpos - (char*)this->file->base));
		
	 	return ((this->mode < 6) ? num : (num - 3));
	
} 
/**
     * Writes a buffer of bytes up to the specified length.
     *
     * @param	buffer the buffer of bytes to be written
     * @param	len the specified length
     */
void TFile_writeBytes_char_p_short_int ( struct File * this , char * buffer , short int len ) { 
if (this->mode > 5) {
			short ind = 0;
			while (len-- > 0) {
				if (fputc(buffer[ind++], this->file) == EOF) {
					this->error = TRUE;
					return ;
				}
				
				this->error = FALSE;
			}
		}
		else {
			this->error = (fwrite(buffer, len, 1, this->file) != 1);
		}
	
} 
/**
     * Copies to a buffer of bytes up to the specified length bytes read from the stream.
     *
     * @param	buffer the buffer of bytes where the bytes will be copied
     * @param	len the specified length
     * @return  the effective number of bytes read.
     */
short int TFile_readBytesCount_char_p_short_int ( struct File * this , char * buffer , short int len ) { 
if (this->mode < 6) {
			this->error = (fread(buffer, len, 1, this->file) != 1) ;
			
			if (!this->error) {
				return len;
			}
			else {
				return 0;
			}
		}
		else {
			long int pos = 0;
			short tmp;
			
			while (len-- > 0) {
				tmp = fgetc(this->file);
				if (tmp == EOF) {
					this->error = TRUE;
					return pos;
				}
				else if (((char)tmp) != '\r') {
					buffer[pos] = (char)tmp;
				}
				else {
					buffer[pos] = '\n';
					len--;
				}
				
				pos ++;
			}
			
			this->error = FALSE;
			return pos;
		}
	
} 
/**
     * Frees the memory of system ressources used by this File
     * object.  If the stream is opened, it will be closed first.
     */
void TFile_finalize_ ( struct File * this ) { 
	if ( this -> opened ) { 
		this -> close_ ( this ) ; 
		
	} 
	if ( this -> name ) { 
		this -> name -> finalize_ ( this -> name ) ; 
		
	} 
	if ( this -> type ) { 
		this -> type -> finalize_ ( this -> type ) ; 
		
	} 
	free(this);
		 
} 
