//Defining class String
/*
 * This file contains the definition of all the methods of the class and the definition of the built in functions related to the class.
 */
/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */
/*
 *  This file is part of Moka API.
 *
 *  Moka API is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka API is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Moka API; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * The <code>String</code> class represents character strings. All
 * string literals in Moka programs, such as <code>"abc"</code>, should be
 * implemented as instances of this class for easy manipulation.
 * <p>
 * Strings are not constant; their values can be changed after they
 * are created.
 * Because String objects are mutable they can't be shared. For example:
 * <p><blockquote><pre>
 *     String str = "abc";
 * </pre></blockquote><p>
 * is not equivalent to:
 * <p><blockquote><pre>
 *     char[] chars = {'a', 'b', 'c', '\0'};
 *     String str = new String(chars);
 * </pre></blockquote><p>
 * The class <code>String</code> includes methods for examining
 * individual characters of the sequence, for comparing strings, for
 * searching strings, for extracting substrings, and for creating a
 * copy of a string with all characters translated to uppercase or to
 * lowercase.
 * <p>
 * String conversions are implemented through the method
 * <code>toString</code>, defined by <code>Object</code> and
 * inherited by all classes in Moka.
 * @author  Frdric Brown
 * @version 1.02, 2002-08-29
 * @since   MDK1.0a
 */
TString * TString0 ( ) {
	TString * this = malloc ( sizeof ( TString ) ) ; 
	this -> class = ( char * ) "String" ; 
	this -> class_id = 13 ; 
	this -> super = ( void * ) Object ; 
	//Methods from class Object
	//Methods from class String
	this -> append_String = ( void * ) TString_append_String ; 
	this -> append_char = ( void * ) TString_append_char ; 
	this -> append_short_int = ( void * ) TString_append_short_int ; 
	this -> concat_String = ( void * ) TString_concat_String ; 
	this -> delete_long_int_long_int = ( void * ) TString_delete_long_int_long_int ; 
	this -> substring_long_int_long_int = ( void * ) TString_substring_long_int_long_int ; 
	this -> toString_ = ( void * ) TString_toString_ ; 
	this -> finalize_ = ( void * ) TString_finalize_ ; 
	return this ; 
} 
/**
     * Initializes a newly created <code>String</code> object so that it
     * represents an empty character sequence.
     */
TString * TString_ ( ) {
	TString * this = TString0 ( ) ; 
	String__ ( this ) ; 
	return this ; 
} 
void String__ ( struct String * this ) { 
this->value = (char*) "";
 		this->ref = TRUE;
 	
} 
/**
     * Initializes a newly created <code>String</code> object so that it
     * represents the same sequence of characters as the argument; in other
     * words, the newly created string is a copy of the argument string.
     *
     * @param   original   a <code>String</code>.
     */
TString * TString_String ( struct String * original ) {
	TString * this = TString0 ( ) ; 
	String__String ( this , original ) ; 
	return this ; 
} 
void String__String ( struct String * this , struct String * original ) { 
	this -> value = Character_copyValueOf_char_p ( original -> value ) ; 
	this -> ref = FALSE ; 
	
} 
/**
     * Initializes a newly created <code>String</code> object so that its
     * value makes reference to a given ANSI string.
     *
     * @param   ansiString   an ANSI string.
     */
TString * TString_char_p ( char * ansiString ) {
	TString * this = TString0 ( ) ; 
	String__char_p ( this , ansiString ) ; 
	return this ; 
} 
void String__char_p ( struct String * this , char * ansiString ) { 
	this -> value = ansiString ; 
	this -> ref = TRUE ; 
	
} 
/**
     * Initializes a newly created <code>String</code> object so that its
     * value makes reference or not to a given ANSI string.
     *
     * @param   ansiString   an ANSI string.
     * @param   ref          true if the value is a reference, false otherwise.
     */
TString * TString_char_p_BOOL ( char * ansiString , BOOL ref ) {
	TString * this = TString0 ( ) ; 
	String__char_p_BOOL ( this , ansiString , ref ) ; 
	return this ; 
} 
void String__char_p_BOOL ( struct String * this , char * ansiString , BOOL ref ) { 
	this -> value = ansiString ; 
	this -> ref = ref ; 
	
} 
/**
     * Appends the string to this string.
     * <p>
     * The characters of the <code>String</code> argument are appended, in
     * order, to the contents of this string, increasing the
     * length of this string buffer by the length of the argument. If the string object
     * risk to be used later,use a copy of the string object
	 * instead of the original, the string being finalized before the method returns.
     *
     * @param   str   a string.
     * @return  a reference to this <code>String</code>.
     */
TString *  TString_append_String ( struct String * this , struct String * str ) { 
	return this -> concat_String ( this , str ) ; 
	
} 
/**
     * Appends the string representation of the <code>char</code>
     * argument to this string.
     * <p>
     * The argument is appended to the contents of this string buffer.
     * The length of this string buffer increases by <code>1</code>.
     *
     * @param   c   a <code>char</code>.
     * @return  a reference to this <code>String</code> object.
     */
TString *  TString_append_char ( struct String * this , char c ) { 
	return this -> concat_String ( this , String_valueOf_char ( c ) ) ; 
	
} 
/**
     * Appends the string representation of the <code>short</code>
     * argument to this string.
     * <p>
     * The argument is converted to a string as if by the method
     * <code>String.valueOf</code>, and the characters of that
     * string are then appended to this string.
     *
     * @param   s   an <code>short</code>.
     * @return  a reference to this <code>String</code> object.
     */
TString *  TString_append_short_int ( struct String * this , short int s ) { 
	return this -> concat_String ( this , String_valueOf_short_int ( s ) ) ; 
	
} 
/**
     * Concatenates the specified string to the end of this string.
     * <p>
     * If the length of the argument string is <code>0</code>, then this
     * <code>String</code> object is returned.  If the value made reference
     * to an ANSI string, this reference is not kept. Otherwise, a the value of this
     * <code>String</code> object is modified, representing a character
     * sequence that is the concatenation of the character sequence
     * represented by this <code>String</code> object and the character
     * sequence represented by the argument string.  In both case, the String to
     * concatenate is finalized.<p>
     *
     * @param   str   the <code>String</code> that is concatenated to the end
     *                of this <code>String</code>.
     * @return  a string that represents the concatenation of this object's
     *          characters followed by the string argument's characters.
     */
TString *  TString_concat_String ( struct String * this , struct String * str ) { 
	long int len0 = ( strlen(str->value)  ) ; 
	char * buffer ; 
	if ( len0 == 0 ) { 
		if ( this -> ref ) { 
			buffer = Character_copyValueOf_char_p ( this -> value ) ; 
			this -> ref = FALSE ; 
			
		} 
		else { 
			buffer = this -> value ; 
			
		} 
		
	} 
	else { 
		buffer = Character_concat_char_p_char_p ( this -> value , str -> value ) ; 
		if ( this -> ref ) { 
			this -> ref = FALSE ; 
			
		} 
		else { 
			free(this->value);
				 
		} 
		
	} 
	str -> finalize_ ( str ) ; 
	this -> value = buffer ; 
	return this ; 
	
} 
/**
     * Removes the characters in a substring of this <code>String</code>.
     * The substring begins at the specified <code>start</code> and extends to
     * the character at index <code>end - 1</code>. If
     * <code>start</code> is equal to <code>end</code>, no changes are made.
     *
     * @param      start  The beginning index, inclusive.
     * @param      end    The ending index, exclusive.
     * @return     This string with the characters of the specified substring removed.
     */
TString *  TString_delete_long_int_long_int ( struct String * this , long int start , long int end ) { 
	long int len = ( strlen(this->value) - (end - start)  ) ; 
	char * buffer = ( malloc(len + 1)  ) ; 
	buffer[0] = 0;
			strncat(buffer, this->value, start);
			strcat(buffer, this->value + end);
		 if ( this -> ref ) { 
		this -> ref = FALSE ; 
		
	} 
	else { 
		free(this->value);
			 
	} 
	this -> value = buffer ; 
	return this ; 
	
} 
/**
     * Returns a new string that is a substring of this string. The
     * substring begins at the specified <code>beginIndex</code> and
     * extends to the character at index <code>endIndex - 1</code>.
     * Thus the length of the substring is <code>endIndex-beginIndex</code>.
     *
     * @param      beginIndex   the beginning index, inclusive.
     * @param      endIndex     the ending index, exclusive.
     * @return     the specified substring.
     */
TString *  TString_substring_long_int_long_int ( struct String * this , long int beginIndex , long int endIndex ) { 
TString* str = TString_();
    	long int len = endIndex-beginIndex;

    	str->value = malloc(len + 1);
		str->ref = FALSE;

		if (len > 0) {
			memcpy(str->value, &this->value[beginIndex], len);
		}

		str->value[len] = '\0';


    	return str;
    
} 
/**
     * Returns a copy of this string.
     *
     * @return  a copy of the string.
     */
TString *  TString_toString_ ( struct String * this ) { 
	return TString_String ( this ) ; 
	
} 
/**
     * Frees the memory of system ressources used by this String
     * object.  If the value of the String do not makes reference to
     * an ANSI String, the ANSI string memory will be freed, otherwise no.
     *
     */
void TString_finalize_ ( struct String * this ) { 
	if ( ! this -> ref ) { 
		free(this->value);
 			 
	} 
	free(this);
 		 
} 
/**
     * Returns the string representation of the <code>char</code> argument.
     *
     * @param   c   an <code>char</code>.
     * @return  a string representation of the <code>char</code> argument.
     */
TString *  String_valueOf_char ( char c ) { 
	char buf[] = {c, '\0'} ;
         return TString_char_p_BOOL ( Character_copyValueOf_char_p ( ( char * ) buf ) , FALSE ) ; 
	
} 
/**
     * Returns the string representation of the <code>short</code> argument.
     *
     * @param   s   an <code>short</code>.
     * @return  a string representation of the <code>short</code> argument.
     */
TString *  String_valueOf_short_int ( short int s ) { 
char vec[7];

		sprintf(vec, "%i", s);

		return TString_char_p_BOOL(strcpy(malloc(strlen(vec) + 1), vec), FALSE);
    
} 
