//Defining class Vector
/*
 * This file contains the definition of all the methods of the class and the definition of the built in functions related to the class.
 */
/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */
/*
 *  This file is part of Moka API.
 *
 *  Moka API is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka API is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Moka API; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * The <code>Vector</code> class implements a growable array of
 * objects. Like an array, it contains components that can be
 * accessed using an integer index. However, the size of a
 * <code>Vector</code> can grow or shrink as needed to accommodate
 * adding and removing items after the <code>Vector</code> has been created.<p>
 *
 * Each vector tries to optimize storage management by maintaining a
 * <code>capacity</code> and a <code>capacityIncrement</code>. The
 * <code>capacity</code> is always at least as large as the vector
 * size; it is usually larger because as components are added to the
 * vector, the vector's storage increases in chunks the size of
 * <code>capacityIncrement</code>. An application can increase the
 * capacity of a vector before inserting a large number of
 * components; this reduces the amount of incremental reallocation. <p>
 *
 * @author  Frdric Brown
 * @version 1.0, 2002-08-29
 * @since   MDK1.0a
 */
TVector * TVector0 ( ) {
	TVector * this = malloc ( sizeof ( TVector ) ) ; 
	this -> class = ( char * ) "Vector" ; 
	this -> class_id = 32 ; 
	this -> super = ( void * ) Object ; 
	//Methods from class Object
	//Methods from class Vector
	this -> push_Object = ( void * ) TVector_push_Object ; 
	this -> finalize_ = ( void * ) TVector_finalize_ ; 
	this -> remove_long_int = ( void * ) TVector_remove_long_int ; 
	this -> toString_ = ( void * ) TVector_toString_ ; 
	return this ; 
} 
/**
 	 * Creates a new Vector with an initial capacity of 10 and
 	 * a capacity increment of 5.
 	 */
TVector * TVector_ ( ) {
	TVector * this = TVector0 ( ) ; 
	Vector__ ( this ) ; 
	return this ; 
} 
void Vector__ ( struct Vector * this ) { 
	Vector__long_int_long_int ( this , 10 , 5 ) ; 
	
} 
/**
 	 * Creates a new Vector with the specified capacity and
 	 * capacity increment.
 	 *
 	 * @param initialCapacity The initial capacity.
 	 * @param capacityIncrement The capacity increment.
 	 */
TVector * TVector_long_int_long_int ( long int initialCapacity , long int capacityIncrement ) {
	TVector * this = TVector0 ( ) ; 
	Vector__long_int_long_int ( this , initialCapacity , capacityIncrement ) ; 
	return this ; 
} 
void Vector__long_int_long_int ( struct Vector * this , long int initialCapacity , long int capacityIncrement ) { 
	this -> capacity = initialCapacity ; 
	this -> capacityIncrement = capacityIncrement ; 
	this -> size = 0 ; 
	this -> array = ( malloc(sizeof(TObject*) * initialCapacity)  ) ; 
	
} 
/**
     * Pushes an item onto the top of this stack.
     *
     * @param   item   the item to be pushed onto this stack.
     */
void TVector_push_Object ( struct Vector * this , struct Object * item ) { 
TObject** array;

 		if (this->size < this->capacity) {
 			this->array[this->size] = item;
 		}
 		else {
 			array = malloc(sizeof(TObject*) * (this->capacity + this->capacityIncrement));
 			this->capacity += this->capacityIncrement;

 			memcpy(array, this->array, (this->size) * sizeof(TObject*));

 			array[this->size] = item;

 			free(this->array);

 			this->array = array;
 		}

		this->size++;
 	
} 
/**
     * Frees the memory used by this object.
     */
void TVector_finalize_ ( struct Vector * this ) { 
free(this->array);
    	free(this);
    
} 
/**
     * Removes the element at the specified position in this vector.
     * Shifts any subsequent elements to the left (subtracts one
     * from their indices).  Returns the element that was removed from the
     * vector.
     *
     * @param index the index of the element to removed.
     * @return the element previously at the specified position.
     */
TObject *  TVector_remove_long_int ( struct Vector * this , long int index ) { 
TObject* o = this->array[index];

    	memmove(this->array + index, this->array + index + 1, (this->size - index - 1) * sizeof(TObject*));
    	this->size--;

    	return o;
    
} 
/**
     * Returns a string representation of this Vector, containing
     * the String representation of each element.
     */
TString *  TVector_toString_ ( struct Vector * this ) { 
	long int i ; 
	TString *  str = TString_char_p ( ( char * ) "[" ) ; 
	for ( i = 0 ; 
	i < this -> size ; 
	i ++ ) { 
		str -> concat_String ( str , TString_char_p ( ( char * ) "[" ) ) ; 
		str -> concat_String ( str , ( TString *  ) this -> array [ i ] -> toString_ ( this -> array [ i ] ) ) ; 
		str -> concat_String ( str , TString_char_p ( ( char * ) "]" ) ) ; 
		
	} 
	str -> concat_String ( str , TString_char_p ( ( char * ) "]" ) ) ; 
	return str ; 
	
} 
